@@ -23,8 +23,9 @@
 #  include <config.h>
 #endif
 
-#include <gnome.h>
 #include <gdk/gdkx.h>
+#include <glib.h>
+#include <glib/gi18n.h>
 #ifdef HAVE_XINPUT
 #include <X11/extensions/XInput.h>
 #endif
