@@ -3,8 +3,8 @@
 *
 * Main file for the mighty GOK
 *
-* Copyright 2001 - 2004 Sun Microsystems, Inc.,
-* Copyright 2001 - 2004 University Of Toronto*
+* Copyright 2001-2009 Sun Microsystems, Inc.,
+* Copyright 2001-2009 University Of Toronto*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
@@ -30,8 +30,11 @@
 #include <sys/types.h>
 #include <dirent.h>
 #include <signal.h>
-#include <gnome.h>
+#include <libgnome/gnome-program.h>
+#include <libgnomeui/gnome-client.h>
+#include <libgnomeui/libgnomeui.h>
 #include <glib/gstdio.h>
+#include <glib/gi18n.h>
 #include <cspi/spi.h>
 #include <libbonobo.h>
 #include <bonobo-activation/bonobo-activation.h>
@@ -173,7 +176,6 @@ static void gok_main_initialize_access_methods (GokArgs *args);
 static void gok_main_initialize_wordcomplete (void);
 static void gok_main_initialize_commandprediction (void);
 static gboolean gok_main_display_scan_main(void);
-static void gok_main_display_geometry_error (void);
 static void gok_main_object_state_listener (Accessible* pAccessible);
 static gboolean gok_main_has_xkb_extension ();
 static gboolean gok_main_check_sticky_keys (GtkWidget *widget);
@@ -564,21 +566,48 @@ gint main (gint argc, gchar *argv[])
 	return 0;
 }
 
+/**
+ * gok_main_utf8_casecmp:
+ * @a: a UTF-8 encoded string
+ * @b: a UTF-8 encoded string
+ *
+ * Compares two strings for ordering using the linguistically correct
+ * rules for the current locale. Case-insensitve version.
+ *
+ * Returns: < 0 if a compares before b, 0 if they compare equal, > 0 if a compares after b.
+ **/
+gint
+gok_main_utf8_casecmp (const gchar *a, const gchar *b)
+{
+	gchar *a_fold, *b_fold;
+	gint res;
+
+	a_fold = g_utf8_casefold (a, -1);
+	b_fold = g_utf8_casefold (b, -1);
+
+	res = g_utf8_collate (a_fold, b_fold);
+
+	g_free (a_fold);
+	g_free (b_fold);
+
+	return res;
+}
+
 /* private helper */
 gint
 comparebasenames (gconstpointer a, gconstpointer b)
 {
-	gchar* a_file;
-	gchar* b_file;
+	gchar *a_file, *b_file;
 	gint returncode;
-	a_file = g_path_get_basename(a);
-	b_file = g_path_get_basename(b);
 
-	returncode = g_strcasecmp(a_file, b_file);
-	
+	a_file = g_path_get_basename (a);
+	b_file = g_path_get_basename (b);
+
+	returncode = gok_main_utf8_casecmp (a_file, b_file);
+
 	g_free (a_file);
 	g_free (b_file);
-	
+
 	return returncode;
 }
 
@@ -694,7 +723,10 @@ gok_main_open(gint argc, gchar *argv[])
 		}
 	    }
 	}
-	
+
+	/* set default icon early so it can be used by error dialogs */
+	gtk_window_set_default_icon_name ("gok");
+
 	/* Parse geometry */
 	if (gok_args.geometry != NULL)
 	{
@@ -725,9 +757,9 @@ gok_main_open(gint argc, gchar *argv[])
 		}
 		else
 		{
-			fprintf (stderr, _("gok: Unsupported geometry specification\n"));
-			fprintf (stderr, _("gok: Currently GOK requires that the x, y, width and height all be given\n"));
-			gok_main_display_geometry_error ();
+			gok_main_display_error (_("Unsupported geometry specification"),
+						_("Currently GOK requires that the x, y, width and height all be given. "
+						  "Sorry, your geometry specification will not be used."));
 		}
 	}
 
@@ -822,7 +854,7 @@ gok_main_open(gint argc, gchar *argv[])
 	/* check for xkb extension */
 	if (!gok_main_has_xkb_extension())
 	{
-		gok_main_display_error (_("XKB extension is required."));
+		gok_main_display_fatal_error (_("XKB extension is required."));
 		return GOK_ERROR_NO_XKB_EXTENSION;
 	}		
 
@@ -832,7 +864,7 @@ gok_main_open(gint argc, gchar *argv[])
 	/* initialize the actions */
 	if (gok_action_open() == FALSE)
 	{
-		gok_main_display_error (_("Can't initialize actions."));
+		gok_main_display_fatal_error (_("Can't initialize actions."));
 		return GOK_ERROR_ACTION_OPEN;
 	}
 
@@ -842,7 +874,7 @@ gok_main_open(gint argc, gchar *argv[])
 	/* initialize the feedbacks */
 	if (gok_feedback_open() == FALSE)
 	{
-		gok_main_display_error (_("Can't initialize feedbacks."));
+		gok_main_display_fatal_error (_("Can't initialize feedbacks."));
 		return GOK_ERROR_FEEDBACK_OPEN;
 	}
 
@@ -887,7 +919,7 @@ gok_main_open(gint argc, gchar *argv[])
 	
 		 if (g_strv_length(vid_pid) != 2) {
 		 	g_strfreev(vid_pid);
-		 	gok_main_display_error(_("can't initialize the libusb backend - can't parse USB VID:PID pair"));
+			gok_main_display_fatal_error(_("Can't initialize the libusb backend - can't parse USB VID:PID pair"));
 		 	return GOK_ERROR_LIBUSB;
 		 }
 
@@ -896,7 +928,7 @@ gok_main_open(gint argc, gchar *argv[])
 		g_strfreev(vid_pid);
 		
 		if ((vid == 0 || pid == 0) && errno == EINVAL) {
-			gok_main_display_error(_("can't initialize the libusb backend - can't parse USB VID:PID pair"));
+			gok_main_display_fatal_error(_("Can't initialize the libusb backend - can't parse USB VID:PID pair"));
 		 	return GOK_ERROR_LIBUSB;
 		}
 		
@@ -912,7 +944,7 @@ gok_main_open(gint argc, gchar *argv[])
 	m_pWindowMain = gok_main_create_window (window_realize_data);
 	if (m_pWindowMain == NULL)
 	{
-		gok_main_display_error (_("Can't create the main GOK window!"));
+		gok_main_display_fatal_error (_("Can't create the main GOK window!"));
 		return GOK_ERROR_MAIN_CREATE_WINDOW;
 	}
 	gtk_widget_show (m_pWindowMain);
@@ -931,7 +963,7 @@ gok_main_open(gint argc, gchar *argv[])
 	/* create the settings dialog */
 	if (gok_settingsdialog_open (gok_args.display_settings_dialog) == FALSE)
 	{
-		gok_main_display_error (_("Can't create the settings dialog window!"));
+		gok_main_display_fatal_error (_("Can't create the settings dialog window!"));
 		return GOK_ERROR_SETTINGSDIALOG_OPEN;
 	}
 
@@ -994,7 +1026,7 @@ gok_main_display_scan_main()
 		/* if no "main" keyboard then display first keyboard in list */
 		if (gok_main_display_scan ((GokKeyboard*)NULL, NULL, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL, KEYBOARD_SHAPE_BEST) == FALSE)
 		{
-			gok_main_display_error (_("No keyboards to display!"));
+			gok_main_display_fatal_error (_("No keyboards to display!"));
 			return FALSE;
 		}
 	}
@@ -1709,7 +1741,7 @@ gok_main_keyboard_find_byname (const gchar *NameKeyboard)
 		pKB = m_pKeyboardFirst;
 		while (pKB != NULL)
 		{
-			if (g_strcasecmp (pKB->Name, NameKeyboard) == 0)
+			if (gok_main_utf8_casecmp (pKB->Name, NameKeyboard) == 0)
 			{
 				/* found the keyboard */
 				break;
@@ -1903,50 +1935,37 @@ GtkWidget* gok_main_create_window (struct gok_window_realize_data *window_realiz
 	gtk_window_set_keep_above (GTK_WINDOW (window1), TRUE); /* please! */
 	
 	g_set_application_name (_("GOK"));
-	gtk_window_set_default_icon_name ("gok");
 
 	m_OurResizeWidth = 200;
 	m_OurResizeHeight = 100;
 	gtk_window_set_default_size (GTK_WINDOW (window1), m_OurResizeWidth, m_OurResizeHeight);
 
 	fixed1 = gtk_fixed_new ();
-	gtk_widget_ref (fixed1);
-	gtk_object_set_data_full (GTK_OBJECT (window1), "fixed1", fixed1,
-                            (GtkDestroyNotify) gtk_widget_unref);
+	g_object_ref (fixed1);
+	g_object_set_data_full (G_OBJECT (window1), "fixed1", fixed1,
+				(GDestroyNotify) g_object_unref);
 	gtk_widget_show (fixed1);
 	gtk_container_add (GTK_CONTAINER (window1), fixed1);
 
-	gtk_signal_connect (GTK_OBJECT (window1), "motion_notify_event",
-                      GTK_SIGNAL_FUNC (on_window1_motion_notify_event),
-                      NULL);
-
-	gtk_signal_connect (GTK_OBJECT (window1), "leave_notify_event",
-                      GTK_SIGNAL_FUNC (on_window1_leave_notify_event),
-                      NULL);
-
-	gtk_signal_connect (GTK_OBJECT (window1), "enter_notify_event",
-                      GTK_SIGNAL_FUNC (on_window1_enter_notify_event),
-                      NULL);
-
 	gtk_widget_add_events (window1, GDK_POINTER_MOTION_MASK);
 
-	gtk_signal_connect (GTK_OBJECT (window1), "size_allocate",
-                      GTK_SIGNAL_FUNC (on_window1_size_allocate),
-                      NULL);
-                      
-	gtk_signal_connect (GTK_OBJECT (window1), "delete_event",
-                      GTK_SIGNAL_FUNC (on_window1_delete_event),
-                      NULL);
-
-	gtk_signal_connect (GTK_OBJECT (window1), "configure_event",
-                      GTK_SIGNAL_FUNC (on_window1_configure_event),
-                      NULL);
-					  
-	gtk_signal_connect (GTK_OBJECT (window1), "window_state_event",
-                      GTK_SIGNAL_FUNC (on_gok_window_state_event),
-                      NULL);
+	g_signal_connect (window1, "motion_notify_event",
+			  G_CALLBACK (on_window1_motion_notify_event), NULL);
+	g_signal_connect (window1, "leave_notify_event",
+			  G_CALLBACK (on_window1_leave_notify_event), NULL);
+	g_signal_connect (window1, "enter_notify_event",
+			  G_CALLBACK (on_window1_enter_notify_event), NULL);
+	g_signal_connect (window1, "size_allocate",
+			  G_CALLBACK (on_window1_size_allocate), NULL);
+	g_signal_connect (window1, "delete_event",
+			  G_CALLBACK (on_window1_delete_event), NULL);
+	g_signal_connect (window1, "configure_event",
+			  G_CALLBACK (on_window1_configure_event), NULL);
+	g_signal_connect (window1, "window_state_event",
+			  G_CALLBACK (on_gok_window_state_event), NULL);
 
 	gok_log_leave();
+
 	return window1;
 }
 
@@ -2087,7 +2106,7 @@ void gok_main_read_keyboards ()
 
     if (m_pKeyboardFirst == NULL)
     {
-	gok_main_display_error (_("Can't create a compose keyboard!"));
+	gok_main_display_fatal_error (_("Can't create a compose keyboard!"));
 	exit (1);
     }
 
@@ -2129,7 +2148,7 @@ void gok_main_read_keyboards ()
     
     if ( gconf_keyboard_directory_error || (m_pKeyboardFirst == NULL) )
     {
-	gok_main_display_error (_("Can't read any keyboards!"));
+	gok_main_display_fatal_error (_("Can't read any keyboards!"));
 	exit (1);
     }
 }
@@ -2151,7 +2170,7 @@ gok_main_read_keyboards_from_dir (const char *directory, GokKeyboard *keyboard)
 	struct dirent* pDirectoryEntry;
 	gchar* filename;
 	gchar* complete_path;
-	GList* langlist = NULL;
+	const gchar * const *langlist;
 	gchar *dirtemp;
 
 	gok_log_enter();
@@ -2159,18 +2178,13 @@ gok_main_read_keyboards_from_dir (const char *directory, GokKeyboard *keyboard)
 	pDirectoryKeyboards = NULL;
 
 	/* find the locale specific keyboards */
-	langlist = (GList*) gnome_i18n_get_language_list(NULL);
-	if (langlist != NULL) {
-		gint i = 0;
-		guint l = g_list_length(langlist);
-		while ((i < l)) {
-			dirtemp = g_build_filename (directory, g_list_nth_data(langlist, i), NULL);
-			pDirectoryKeyboards = opendir (dirtemp);
-			if (pDirectoryKeyboards) 
-				break;
-			g_free (dirtemp);
-			i++;
-		}
+	langlist = g_get_language_names ();
+	while (*langlist) {
+		dirtemp = g_build_filename (directory, *langlist++, NULL);
+		pDirectoryKeyboards = opendir (dirtemp);
+		if (pDirectoryKeyboards) 
+			break;
+		g_free (dirtemp);
 	}
 
 	if (pDirectoryKeyboards == NULL) { /* no locale specific directory found */
@@ -2244,22 +2258,18 @@ gok_main_initialize_access_methods (GokArgs *args)
     else if (gconf_err != NULL) {
 		gok_log_x ("Error getting access method directory key from GConf");
 		g_error_free (gconf_err);
- 		gok_main_display_error
-			(_("could not access method directory key from GConf!"));
+		gok_main_display_fatal_error (
+			_("Could not get access method directory key from GConf."));
 		exit(2);		
     }
     else if (!(gok_scanner_initialize(directory_name, 
 				      args->accessmethodname, 
 				      args->selectactionname, 
 				      args->scanactionname))) {
- 		gok_main_display_error
-			(_("possibly unknown access method!"));
+		gok_main_display_fatal_error (_("Possibly unknown access method."));
 		exit(2);
-			
     }
-    
     g_free (directory_name);
-    
 }
 
 /**
@@ -2707,7 +2717,7 @@ void gok_main_store_window_center ()
 
 	/* get the center of the current window */	
 	gdk_window_get_position (m_pWindowMain->window, &winX, &winY);
-	gdk_window_get_size (m_pWindowMain->window, &winWidthCurrent, &winHeightCurrent);
+	gdk_drawable_get_size (m_pWindowMain->window, &winWidthCurrent, &winHeightCurrent);
 	winX +=  winWidthCurrent / 2;	
 	winY +=  winHeightCurrent / 2;
 
@@ -2785,7 +2795,6 @@ gok_main_close()
 	gok_data_close();
 
 	closeSwitchApi();
-	gok_sound_shutdown();
 	gok_modifier_close();
 	gok_action_close();
 	gok_feedback_close();
@@ -2797,65 +2806,96 @@ gok_main_close()
 }
 
 /**
-* gok_main_display_error
-* @ErrorString: Fatal error message.
-* 
-* Displays a fatal error dialog (modal).
-*/
-void gok_main_display_error (gchar* ErrorString)
+ * gok_main_display_fatal_error:
+ * @ErrorString: Fatal error message.
+ *
+ * Displays a fatal error dialog (modal).
+ */
+void
+gok_main_display_fatal_error (const gchar *ErrorString)
 {
-	GtkWidget* pDialog;
-	gchar* buffer;
-	
-	buffer = g_strdup_printf (_("Sorry, GOK can't run because:\n%s"), ErrorString);
-	
+	GtkWidget* pDialog, *main;
+
+	g_return_if_fail (ErrorString != NULL);
+
 	pDialog = gtk_message_dialog_new (NULL,
-		GTK_DIALOG_DESTROY_WITH_PARENT,
-		GTK_MESSAGE_ERROR,
-		GTK_BUTTONS_CLOSE,
-		"%s", buffer);
-	
+					  GTK_DIALOG_DESTROY_WITH_PARENT,
+					  GTK_MESSAGE_ERROR,
+					  GTK_BUTTONS_CLOSE,
+					  "%s", _("Sorry, GOK can't run"));
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (pDialog),
+						  "%s", ErrorString);
+
+	/* use the same screen as the main window */
+	main = gok_main_get_main_window ();
+	if (main)
+	{
+		gtk_window_set_screen (GTK_WINDOW (pDialog),
+				       gtk_widget_get_screen (main));
+	}
 	gtk_window_set_title (GTK_WINDOW (pDialog), _("GOK Fatal Error"));
 	gtk_dialog_run (GTK_DIALOG (pDialog));
 	gtk_widget_destroy (pDialog);
-	g_free (buffer);
 }
 
 /**
-* gok_main_display_gconf_error
-* 
-* Displays a gconf error message.
-*/
-void gok_main_display_gconf_error ()
+ * gok_main_display_error:
+ * @error_msg: Error message
+ * @detail: Error details
+ *
+ * Displays an error dialog.
+ */
+void
+gok_main_display_error (const gchar *error,
+			const gchar *detail)
 {
-    gok_main_display_error ( _("GOK uses GConf 2 to store its settings and requires certain settings to be in GConf to run.  GOK is currently unable to retrieve those settings.  If this is the first time that you have run gok after installing it you may need to restart gconfd, you can use this command: 'gconftool-2 --shutdown' or log out and back in."));
-}
+	GtkWidget *dialog, *main;
 
-/**
-* gok_main_display_geometry_error
-* 
-* Displays a dialog informing user that the given geometry was not correct.
-*/
-static void
-gok_main_display_geometry_error ()
-{
-	GtkWidget* dialog;
-        
-	dialog = gtk_message_dialog_new (NULL,
-	                                 GTK_DIALOG_DESTROY_WITH_PARENT,
-	                                 GTK_MESSAGE_ERROR,
-	                                 GTK_BUTTONS_CLOSE,
-	                                 _("Currently GOK requires that the x, y, width and height all be given.  Sorry, your geometry specification will not be used."));
-        
-	gtk_window_set_title (GTK_WINDOW (dialog), _("gok: Unsupported geometry specification"));
+	g_return_if_fail (error != NULL);
 
-	g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
-	                          G_CALLBACK (gtk_widget_destroy),
-	                          GTK_OBJECT (dialog));
+	dialog = gtk_message_dialog_new (NULL, 
+					 GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+					 "%s", error);
+	if (detail)
+	{
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+							  "%s", detail);
+	}
+	/* use the same screen as the main window */
+	main = gok_main_get_main_window ();
+	if (main)
+	{
+		gtk_window_set_screen (GTK_WINDOW (dialog),
+				       gtk_widget_get_screen (main));
+	}
+	gtk_window_set_title (GTK_WINDOW (dialog), _("GOK Error"));
+	/* avoid self occlusion */
+	gtk_window_set_type_hint (GTK_WINDOW (dialog),
+				  GDK_WINDOW_TYPE_HINT_NORMAL);
+	gtk_window_set_position (GTK_WINDOW (dialog),
+				 GTK_WIN_POS_CENTER);
 
+	g_signal_connect_swapped (dialog, "response",
+				  G_CALLBACK (gtk_widget_destroy), dialog);
 	gtk_widget_show_all (dialog);
 }
 
+/**
+ * gok_main_display_gconf_error:
+ *
+ * Displays a gconf error message.
+ */
+void
+gok_main_display_gconf_error (void)
+{
+	gok_main_display_fatal_error (_("GOK uses GConf 2 to store its settings and requires certain settings "
+					"to be in GConf to run. GOK is currently unable to retrieve those settings. "
+					"If this is the first time that you have run gok after installing it you "
+					"may need to restart gconfd, you can use this command: "
+					"'gconftool-2 --shutdown' or log out and back in."));
+}
+
 /**
 * gok_main_get_use_geometry
 * 
@@ -2894,7 +2934,7 @@ gok_main_window_contains_pointer (void)
 	  gdk_window_get_position (widget->window, &x, &y);
 	  x = px-x;
 	  y = py-y;
-	  gdk_window_get_size (widget->window, &w, &h);
+	  gdk_drawable_get_size (widget->window, &w, &h);
 	}
 	if (x < 0 || y < 0 || x > w || y > h)
 	  return FALSE;
@@ -3112,16 +3152,8 @@ gok_main_check_sticky_keys (GtkWidget *widget)
 	}
 	else {
 		if (!gok_args.is_login) {
-                /* post an error dialog */
-			dialog = gtk_message_dialog_new (NULL,
-						 GTK_DIALOG_DESTROY_WITH_PARENT,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("GOK cannot run because XKB display extension is missing.\n"));
-			g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
-					  G_CALLBACK (gtk_widget_destroy),
-					  GTK_OBJECT (dialog));
-			gtk_widget_show_all (dialog);
+			/* post an error dialog */
+			gok_main_display_fatal_error (_("XKB display extension is missing."));
 		}
 		else
 			g_warning ("XKB extension not present or non-functional.  GOK will exit.");
@@ -3165,15 +3197,49 @@ void check_accessibility_cb ( GObject* o, gpointer* data )
 
 }
 
+/**
+ * gok_main_display_help:
+ * @doc_name: name of the help document
+ * @link_id: id of the section to display or NULL
+ *
+ * Displays the GOK Help text in the gnome-help browser.
+ **/
 void
-gok_main_help_cb (gpointer data) 
+gok_main_display_help (const gchar *doc_name, const gchar *link_id)
 {
+	GtkWidget *main;
+	GdkScreen *screen;
+	gchar *uri;
 	GError *error = NULL;
-	gchar *helpuri = data;
-	/* TODO: detect error launching help, and give informative message */
-	/* TODO: remove hack for link below, if and when this method is called from more than one dialog */
 
-	gnome_help_display_desktop (NULL, "gnome-access-guide", helpuri, "sysadmin-37", &error);
+	g_return_if_fail (doc_name != NULL);
+
+	if (link_id)
+	{
+		uri = g_strdup_printf ("ghelp:%s?%s", doc_name, link_id);
+	}
+	else
+	{
+		uri = g_strdup_printf ("ghelp:%s", doc_name);
+	}
+
+	main = gok_main_get_main_window ();
+	screen = gtk_widget_get_screen (main);
+	gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error);
+	g_free (uri);
+
+	if (error)
+	{
+		gok_main_display_error (_("Unable to open help file"),
+		                        error->message);
+		g_error_free (error);
+	}
+}
+
+static void
+gok_main_help_cb (GtkButton *button, gpointer data)
+{
+	gok_main_display_help ("gnome-access-guide", "sysadmin-37");
 }
 
 void
@@ -3247,8 +3313,8 @@ gok_main_warn (gchar *message, gboolean always, GCallback cancelfunc, GCallback
 			button = gtk_button_new_from_stock(GTK_STOCK_HELP);
 			gtk_container_add (GTK_CONTAINER (GTK_DIALOG(_corepointer_warning)->action_area),
 					   button);
-			g_signal_connect_swapped (G_OBJECT (button), "clicked", 
-						  G_CALLBACK (gok_main_help_cb), "sysadmin.xml");
+			g_signal_connect (button, "clicked",
+					  G_CALLBACK (gok_main_help_cb), NULL);
 			
 			button = gtk_button_new_from_stock(GTK_STOCK_PREFERENCES);
 			gtk_container_add (GTK_CONTAINER (GTK_DIALOG(_corepointer_warning)->action_area),
